@@ -12,7 +12,7 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
     Button
 from trytond.report import Report
-from trytond.tools import reduce_ids
+from trytond.tools import reduce_ids, grouped_slice
 from trytond.pyson import Eval, PYSONEncoder, Date
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -372,6 +372,11 @@ class AccountTemplate(ModelSQL, ModelView):
     deferral = fields.Boolean('Deferral', states={
             'invisible': Eval('kind') == 'view',
             }, depends=['kind'])
+    party_required = fields.Boolean('Party Required',
+        states={
+            'invisible': Eval('kind') == 'view',
+            },
+        depends=['kind'])
 
     @classmethod
     def __setup__(cls):
@@ -396,6 +401,10 @@ class AccountTemplate(ModelSQL, ModelView):
     def default_deferral():
         return True
 
+    @staticmethod
+    def default_party_required():
+        return False
+
     def get_rec_name(self, name):
         if self.code:
             return self.code + ' - ' + self.name
@@ -404,12 +413,10 @@ class AccountTemplate(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        templates = cls.search([
-                ('code',) + tuple(clause[1:]),
-                ], limit=1)
-        if templates:
-            return [('code',) + tuple(clause[1:])]
-        return [(cls._rec_name,) + tuple(clause[1:])]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     def _get_account_value(self, account=None):
         '''
@@ -426,6 +433,8 @@ class AccountTemplate(ModelSQL, ModelView):
             res['reconcile'] = self.reconcile
         if not account or account.deferral != self.deferral:
             res['deferral'] = self.deferral
+        if not account or account.party_required != self.party_required:
+            res['party_required'] = self.party_required
         if not account or account.template != self:
             res['template'] = self.id
         return res
@@ -584,6 +593,11 @@ class Account(ModelSQL, ModelView):
         'Deferrals', readonly=True, states={
             'invisible': Eval('kind') == 'view',
             }, depends=['kind'])
+    party_required = fields.Boolean('Party Required',
+        states={
+            'invisible': Eval('kind') == 'view',
+            },
+        depends=['kind'])
     template = fields.Many2One('account.account.template', 'Template')
 
     @classmethod
@@ -629,6 +643,10 @@ class Account(ModelSQL, ModelView):
     def default_deferral():
         return True
 
+    @staticmethod
+    def default_party_required():
+        return False
+
     @staticmethod
     def default_kind():
         return 'view'
@@ -645,7 +663,6 @@ class Account(ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
         FiscalYear = pool.get('account.fiscalyear')
         cursor = Transaction().cursor
-        in_max = cursor.IN_MAX
 
         table_a = cls.__table__()
         table_c = cls.__table__()
@@ -653,8 +670,7 @@ class Account(ModelSQL, ModelView):
         ids = [a.id for a in accounts]
         balances = dict((i, 0) for i in ids)
         line_query, fiscalyear_ids = MoveLine.query_get(line)
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
+        for sub_ids in grouped_slice(ids):
             red_sql = reduce_ids(table_a.id, sub_ids)
             cursor.execute(*table_a.join(table_c,
                     condition=(table_c.left >= table_a.left)
@@ -674,6 +690,10 @@ class Account(ModelSQL, ModelView):
                 break
             balances[account_id] = Decimal(str(balance))
 
+        for account in accounts:
+            balances[account.id] = account.company.currency.round(
+                balances[account.id])
+
         fiscalyears = FiscalYear.browse(fiscalyear_ids)
         func = lambda accounts, names: \
             {names[0]: cls.get_balance(accounts, names[0])}
@@ -691,7 +711,6 @@ class Account(ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
         FiscalYear = pool.get('account.fiscalyear')
         cursor = Transaction().cursor
-        in_max = cursor.IN_MAX
 
         result = {}
         ids = [a.id for a in accounts]
@@ -706,8 +725,7 @@ class Account(ModelSQL, ModelView):
         columns = [table.id]
         for name in names:
             columns.append(Sum(Coalesce(Column(line, name), 0)))
-        for i in range(0, len(ids), in_max):
-            sub_ids = ids[i:i + in_max]
+        for sub_ids in grouped_slice(ids):
             red_sql = reduce_ids(table.id, sub_ids)
             cursor.execute(*table.join(line, 'LEFT',
                     condition=line.account == table.id
@@ -722,6 +740,10 @@ class Account(ModelSQL, ModelView):
                         result[name][account_id] = Decimal(str(row[i]))
                     else:
                         result[name][account_id] = row[i]
+        for account in accounts:
+            for name in names:
+                result[name][account.id] = account.company.currency.round(
+                    result[name][account.id])
 
         if not Transaction().context.get('cumulate'):
             return result
@@ -739,7 +761,6 @@ class Account(ModelSQL, ModelView):
         pool = Pool()
         FiscalYear = pool.get('account.fiscalyear')
         Deferral = pool.get('account.account.deferral')
-        in_max = Transaction().cursor.IN_MAX
         names = values.keys()
 
         youngest_fiscalyear = None
@@ -764,11 +785,10 @@ class Account(ModelSQL, ModelView):
         if fiscalyear.state == 'close':
             id2deferral = {}
             ids = [a.id for a in accounts]
-            for i in range(0, len(ids), in_max):
-                sub_ids = ids[i:i + in_max]
+            for sub_ids in grouped_slice(ids):
                 deferrals = Deferral.search([
                     ('fiscalyear', '=', fiscalyear.id),
-                    ('account', 'in', sub_ids),
+                    ('account', 'in', list(sub_ids)),
                     ])
                 for deferral in deferrals:
                     id2deferral[deferral.account.id] = deferral
@@ -797,12 +817,10 @@ class Account(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        accounts = cls.search([
-                ('code',) + tuple(clause[1:]),
-                ], limit=1)
-        if accounts:
-            return [('code',) + tuple(clause[1:])]
-        return [(cls._rec_name,) + tuple(clause[1:])]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     @classmethod
     def copy(cls, accounts, default=None):
@@ -817,18 +835,23 @@ class Account(ModelSQL, ModelView):
         return new_accounts
 
     @classmethod
-    def write(cls, accounts, vals):
-        if not vals.get('active', True):
-            MoveLine = Pool().get('account.move.line')
-            childs = cls.search([
-                    ('parent', 'child_of', [a.id for a in accounts]),
-                    ])
-            if MoveLine.search([
-                        ('account', 'in', [a.id for a in childs]),
-                        ]):
-                vals = vals.copy()
-                del vals['active']
-        super(Account, cls).write(accounts, vals)
+    def write(cls, *args):
+        pool = Pool()
+        MoveLine = pool.get('account.move.line')
+        actions = iter(args)
+        args = []
+        for accounts, values in zip(actions, actions):
+            if not values.get('active', True):
+                childs = cls.search([
+                        ('parent', 'child_of', [a.id for a in accounts]),
+                        ])
+                if MoveLine.search([
+                            ('account', 'in', [a.id for a in childs]),
+                            ]):
+                    values = values.copy()
+                    del values['active']
+            args.extend((accounts, values))
+        super(Account, cls).write(*args)
 
     @classmethod
     def delete(cls, accounts):
@@ -975,14 +998,13 @@ class AccountDeferral(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        deferrals = cls.search(['OR',
-                ('account.rec_name',) + tuple(clause[1:]),
-                ('fiscalyear.rec_name',) + tuple(clause[1:]),
-                ])
-        return [('id', 'in', [d.id for d in deferrals])]
+        return ['OR',
+            ('account.rec_name',) + tuple(clause[1:]),
+            ('fiscalyear.rec_name',) + tuple(clause[1:]),
+            ]
 
     @classmethod
-    def write(cls, deferrals, vals):
+    def write(cls, deferrals, values, *args):
         cls.raise_user_error('write_deferral')
 
 
@@ -1028,7 +1050,7 @@ class PrintGeneralLedgerStart(ModelView):
     'Print General Ledger'
     __name__ = 'account.print_general_ledger.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1063,6 +1085,7 @@ class PrintGeneralLedgerStart(ModelView):
     def default_empty_account():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
@@ -1172,8 +1195,8 @@ class GeneralLedger(Report):
         if not data['empty_account']:
             account2lines = dict(cls.get_lines(accounts,
                     end_periods, data['posted']))
-            for account in (set(accounts) - set(account2lines)):
-                accounts.remove(account)
+            accounts = Account.browse([a.id for a in accounts
+                    if a in account2lines])
 
         account_id2lines = cls.lines(accounts,
             list(set(end_periods).difference(set(start_periods))),
@@ -1242,8 +1265,7 @@ class PrintTrialBalanceStart(ModelView):
     'Print Trial Balance'
     __name__ = 'account.print_trial_balance.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'],
-            depends=['start_period', 'end_period'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1279,6 +1301,7 @@ class PrintTrialBalanceStart(ModelView):
     def default_empty_account():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
@@ -1382,17 +1405,17 @@ class TrialBalance(Report):
                 posted=data['posted']):
             end_accounts = Account.browse(accounts)
 
-        to_remove = []
+        to_remove = set()
         if not data['empty_account']:
             for account in in_accounts:
                 if account.debit == Decimal('0.0') \
                         and account.credit == Decimal('0.0'):
-                    to_remove.append(account.id)
+                    to_remove.add(account)
 
         accounts = []
         for start_account, in_account, end_account in izip(
                 start_accounts, in_accounts, end_accounts):
-            if in_account.id in to_remove:
+            if in_account in to_remove:
                 continue
             accounts.append({
                     'code': start_account.code,
@@ -1486,8 +1509,7 @@ class OpenIncomeStatementStart(ModelView):
     'Open Income Statement'
     __name__ = 'account.open_income_statement.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'],
-            depends=['start_period', 'end_period'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1517,11 +1539,12 @@ class OpenIncomeStatementStart(ModelView):
     def default_posted():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
             'end_period': None,
-        }
+            }
 
 
 class OpenIncomeStatement(Wizard):
@@ -1578,10 +1601,7 @@ class CreateChartStart(ModelView):
 class CreateChartAccount(ModelView):
     'Create Chart'
     __name__ = 'account.create_chart.account'
-    company = fields.Many2One('company.company', 'Company', required=True,
-        domain=[
-            ('parent', 'child_of', [Eval('context', {}).get('company', 0)]),
-            ])
+    company = fields.Many2One('company.company', 'Company', required=True)
     account_template = fields.Many2One('account.account.template',
             'Account Template', required=True, domain=[('parent', '=', None)])
 
@@ -1719,15 +1739,14 @@ class CreateChart(Wizard):
                     ('res', '=', None),
                     ('company', '=', self.properties.company.id),
                     ])
-            with Transaction().set_user(0):
-                Property.delete(properties)
-                if self.properties.account_receivable:
-                    Property.create([{
-                                'field': account_receivable_field.id,
-                                'value': str(
-                                    self.properties.account_receivable),
-                                'company': self.properties.company.id,
-                                }])
+            Property.delete(properties)
+            if self.properties.account_receivable:
+                Property.create([{
+                            'field': account_receivable_field.id,
+                            'value': str(
+                                self.properties.account_receivable),
+                            'company': self.properties.company.id,
+                            }])
 
             account_payable_field, = ModelField.search([
                     ('model.model', '=', 'party.party'),
@@ -1738,14 +1757,13 @@ class CreateChart(Wizard):
                     ('res', '=', None),
                     ('company', '=', self.properties.company.id),
                     ])
-            with Transaction().set_user(0):
-                Property.delete(properties)
-                if self.properties.account_payable:
-                    Property.create([{
-                                'field': account_payable_field.id,
-                                'value': str(self.properties.account_payable),
-                                'company': self.properties.company.id,
-                                }])
+            Property.delete(properties)
+            if self.properties.account_payable:
+                Property.create([{
+                            'field': account_payable_field.id,
+                            'value': str(self.properties.account_payable),
+                            'company': self.properties.company.id,
+                            }])
         return 'end'
 
 
