@@ -2,26 +2,36 @@
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 import datetime
-from itertools import groupby
+from itertools import groupby, combinations
 from operator import itemgetter
+from collections import defaultdict
+
+try:
+    from sql import Null
+except ImportError:
+    Null = None
 from sql.aggregate import Sum
-from sql.conditionals import Coalesce
+from sql.conditionals import Coalesce, Case
 
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateTransition, StateView, StateAction, \
     Button
 from trytond.report import Report
 from trytond import backend
-from trytond.pyson import Eval, PYSONEncoder
+from trytond.pyson import Eval, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.rpc import RPC
-from trytond.tools import reduce_ids
+from trytond.tools import reduce_ids, grouped_slice
+from trytond.config import config
 
 __all__ = ['Move', 'Reconciliation', 'Line', 'OpenJournalAsk',
-    'OpenJournal', 'OpenAccount', 'ReconcileLinesWriteOff', 'ReconcileLines',
-    'UnreconcileLinesStart', 'UnreconcileLines', 'OpenReconcileLinesStart',
-    'OpenReconcileLines', 'FiscalYearLine', 'FiscalYear2',
+    'OpenJournal', 'OpenAccount',
+    'ReconcileLinesWriteOff', 'ReconcileLines',
+    'UnreconcileLines',
+    'Reconcile', 'ReconcileShow',
+    'CancelMoves',
+    'FiscalYearLine', 'FiscalYear2',
     'PrintGeneralJournalStart', 'PrintGeneralJournal', 'GeneralJournal']
 __metaclass__ = PoolMeta
 
@@ -47,7 +57,7 @@ class Move(ModelSQL, ModelView):
     journal = fields.Many2One('account.journal', 'Journal', required=True,
             states=_MOVE_STATES, depends=_MOVE_DEPENDS)
     date = fields.Date('Effective Date', required=True, states=_MOVE_STATES,
-        depends=_MOVE_DEPENDS, on_change_with=['period', 'journal', 'date'])
+        depends=_MOVE_DEPENDS)
     post_date = fields.Date('Post Date', readonly=True)
     description = fields.Char('Description', states=_MOVE_STATES,
         depends=_MOVE_DEPENDS)
@@ -86,6 +96,9 @@ class Move(ModelSQL, ModelView):
                     'because it\'s date is outside its period.'),
                 'draft_closed_period': ('You can not set to draft move '
                     '"%(move)s" because period "%(period)s" is closed.'),
+                'period_cancel': (
+                    'The period of move "%s" is closed.\n'
+                    'Use the current period?'),
                 })
         cls._buttons.update({
                 'post': {
@@ -139,6 +152,7 @@ class Move(ModelSQL, ModelView):
             return period.start_date
         return Date.today()
 
+    @fields.depends('period', 'journal', 'date')
     def on_change_with_date(self):
         Line = Pool().get('account.move.line')
         date = self.date
@@ -155,7 +169,7 @@ class Move(ModelSQL, ModelView):
     @classmethod
     def _get_origin(cls):
         'Return list of Model names for origin Reference'
-        return ['account.fiscalyear']
+        return ['account.fiscalyear', 'account.move']
 
     @classmethod
     def get_origin(cls):
@@ -197,22 +211,27 @@ class Move(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        moves = cls.search(['OR',
-                ('post_number',) + tuple(clause[1:]),
-                (cls._rec_name,) + tuple(clause[1:]),
-                ])
-        return [('id', 'in', [m.id for m in moves])]
+        return ['OR',
+            ('post_number',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     @classmethod
-    def write(cls, moves, vals):
-        keys = vals.keys()
-        for key in cls._check_modify_exclude:
-            if key in keys:
-                keys.remove(key)
-        if len(keys):
-            cls.check_modify(moves)
-        super(Move, cls).write(moves, vals)
-        cls.validate_move(moves)
+    def write(cls, *args):
+        actions = iter(args)
+        all_moves = []
+        args = []
+        for moves, values in zip(actions, actions):
+            keys = values.keys()
+            for key in cls._check_modify_exclude:
+                if key in keys:
+                    keys.remove(key)
+            if len(keys):
+                cls.check_modify(moves)
+            args.extend((moves, values))
+            all_moves.extend(moves)
+        super(Move, cls).write(*args)
+        cls.validate_move(all_moves)
 
     @classmethod
     def create(cls, vlist):
@@ -282,9 +301,7 @@ class Move(ModelSQL, ModelView):
         company = User(user).company
         amounts = {}
         move2draft_lines = {}
-        for i in range(0, len(moves), cursor.IN_MAX):
-            sub_moves = moves[i:i + cursor.IN_MAX]
-            sub_move_ids = [m.id for m in sub_moves]
+        for sub_move_ids in grouped_slice([m.id for m in moves]):
             red_sql = reduce_ids(line.move, sub_move_ids)
 
             cursor.execute(*line.select(line.move,
@@ -321,8 +338,7 @@ class Move(ModelSQL, ModelView):
                 (draft_moves, 'draft'),
                 ):
             if move_ids:
-                for i in range(0, len(move_ids), cursor.IN_MAX):
-                    sub_ids = move_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(move_ids):
                     red_sql = reduce_ids(line.move, sub_ids)
                     # Use SQL to prevent double validate loop
                     cursor.execute(*line.update(
@@ -330,6 +346,50 @@ class Move(ModelSQL, ModelView):
                             values=[state],
                             where=red_sql))
 
+    def _cancel_default(self):
+        'Return default dictionary to cancel move'
+        pool = Pool()
+        Date = pool.get('ir.date')
+        Period = pool.get('account.period')
+
+        default = {
+            'origin': str(self),
+            }
+        if self.period.state == 'close':
+            self.raise_user_warning('%s.cancel' % self,
+                'period_cancel', self.rec_name)
+            date = Date.today()
+            period_id = Period.find(self.company.id, date=date)
+            default.update({
+                    'date': date,
+                    'period': period_id,
+                    })
+        return default
+
+    def cancel(self):
+        'Return a cancel move'
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        TaxLine = pool.get('account.tax.line')
+        default = self._cancel_default()
+        cancel_move, = self.copy([self], default=default)
+        lines = []
+        tax_lines = []
+        for line in cancel_move.lines:
+            line.debit *= -1
+            line.credit *= -1
+            lines.extend(([line], line._save_values))
+            line._values = None
+            for tax_line in line.tax_lines:
+                tax_line.amount *= -1
+                tax_lines.extend(([tax_line], tax_line._save_values))
+                tax_line._values = None
+        if lines:
+            Line.write(*lines)
+        if tax_lines:
+            TaxLine.write(*tax_lines)
+        return cancel_move
+
     @classmethod
     @ModelView.button
     def post(cls, moves):
@@ -427,7 +487,7 @@ class Reconciliation(ModelSQL, ModelView):
         return super(Reconciliation, cls).create(vlist)
 
     @classmethod
-    def write(cls, moves, vals):
+    def write(cls, moves, values, *args):
         cls.raise_user_error('modify')
 
     @classmethod
@@ -442,7 +502,8 @@ class Reconciliation(ModelSQL, ModelView):
             debit = Decimal('0.0')
             credit = Decimal('0.0')
             account = None
-            party = None
+            if reconciliation.lines:
+                party = reconciliation.lines[0].party
             for line in reconciliation.lines:
                 if line.state != 'valid':
                     cls.raise_user_error('reconciliation_line_not_valid',
@@ -463,9 +524,7 @@ class Reconciliation(ModelSQL, ModelView):
                             'line': line.rec_name,
                             'account': line.account.rec_name,
                             })
-                if not party:
-                    party = line.party
-                elif line.party and party.id != line.party.id:
+                if line.party != party:
                     cls.raise_user_error('reconciliation_different_parties', {
                             'line': line.rec_name,
                             'party1': line.party.rec_name,
@@ -492,19 +551,13 @@ class Line(ModelSQL, ModelView):
     __name__ = 'account.move.line'
     debit = fields.Numeric('Debit', digits=(16, Eval('currency_digits', 2)),
         required=True,
-        on_change=['account', 'debit', 'credit', 'tax_lines', 'journal',
-            'move'],
         depends=['currency_digits', 'credit', 'tax_lines', 'journal'])
     credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
         required=True,
-        on_change=['account', 'debit', 'credit', 'tax_lines', 'journal',
-            'move'],
         depends=['currency_digits', 'debit', 'tax_lines', 'journal'])
     account = fields.Many2One('account.account', 'Account', required=True,
             domain=[('kind', '!=', 'view')],
-            select=True,
-            on_change=['account', 'debit', 'credit', 'tax_lines',
-                'journal', 'move'])
+            select=True)
     move = fields.Many2One('account.move', 'Move', select=True, required=True,
         states={
             'required': False,
@@ -524,16 +577,30 @@ class Line(ModelSQL, ModelView):
             selection='get_origin'),
         'get_move_field', searcher='search_move_field')
     description = fields.Char('Description')
+    move_description = fields.Function(fields.Char('Move Description'),
+        'get_move_field', setter='set_move_field',
+        searcher='search_move_field')
     amount_second_currency = fields.Numeric('Amount Second Currency',
             digits=(16, Eval('second_currency_digits', 2)),
             help='The amount expressed in a second currency',
-            depends=['second_currency_digits'])
+        states={
+            'required': Bool(Eval('second_currency')),
+            },
+        depends=['second_currency_digits', 'second_currency'])
     second_currency = fields.Many2One('currency.currency', 'Second Currency',
-            help='The second currency')
-    party = fields.Many2One('party.party', 'Party',
-            on_change=['move', 'party', 'account', 'debit', 'credit',
-                'journal'], select=True, depends=['debit', 'credit', 'account',
-                    'journal'])
+            help='The second currency',
+        states={
+            'required': Bool(Eval('amount_second_currency')),
+            },
+        depends=['amount_second_currency'])
+    party = fields.Many2One('party.party', 'Party', select=True,
+        states={
+            'required': Eval('party_required', False),
+            'invisible': ~Eval('party_required', False),
+            },
+        depends=['party_required'])
+    party_required= fields.Function(fields.Boolean('Party Required'),
+        'on_change_with_party_required')
     maturity_date = fields.Date('Maturity Date',
         help='This field is used for payable and receivable lines. \n'
         'You can put the limit date for the payment.')
@@ -556,7 +623,10 @@ class Line(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Line, cls).__setup__()
-        cls._check_modify_exclude = ['reconciliation']
+        cls._check_modify_exclude = {'reconciliation'}
+        cls._reconciliation_modify_disallow = {
+            'account', 'debit', 'credit', 'party',
+            }
         cls._sql_constraints += [
             ('credit_debit',
                 'CHECK(credit * debit = 0.0)',
@@ -581,6 +651,8 @@ class Line(ModelSQL, ModelView):
                 'move_inactive_account': ('You can not create a move line '
                     'with account "%s" because it is inactive.'),
                 'already_reconciled': 'Line "%s" (%d) already reconciled.',
+                'party_required': 'Party is required on line "%s"',
+                'party_set': 'Party must not be set on line "%s"',
                 })
 
     @classmethod
@@ -838,6 +910,8 @@ class Line(ModelSQL, ModelView):
         Move = Pool().get('account.move')
         return Move.get_origin()
 
+    @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
+        'move')
     def on_change_debit(self):
         changes = {}
         Journal = Pool().get('account.journal')
@@ -851,6 +925,8 @@ class Line(ModelSQL, ModelView):
             changes['credit'] = Decimal('0.0')
         return changes
 
+    @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
+        'move')
     def on_change_credit(self):
         changes = {}
         Journal = Pool().get('account.journal')
@@ -864,6 +940,8 @@ class Line(ModelSQL, ModelView):
             changes['debit'] = Decimal('0.0')
         return changes
 
+    @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
+        'move')
     def on_change_account(self):
         Journal = Pool().get('account.journal')
 
@@ -882,6 +960,12 @@ class Line(ModelSQL, ModelView):
                     self.account.second_currency.digits
         return changes
 
+    @fields.depends('account')
+    def on_change_with_party_required(self, name=None):
+        if self.account:
+            return self.account.party_required
+        return False
+
     def _compute_tax_lines(self, journal_type):
         res = {}
         pool = Pool()
@@ -934,6 +1018,7 @@ class Line(ModelSQL, ModelView):
                     res.setdefault('add', []).append(value)
         return res
 
+    @fields.depends('move', 'party', 'account', 'debit', 'credit', 'journal')
     def on_change_party(self):
         Journal = Pool().get('account.journal')
         cursor = Transaction().cursor
@@ -1032,22 +1117,20 @@ class Line(ModelSQL, ModelView):
         return res
 
     def get_move_field(self, name):
-        if name == 'move_state':
-            name = 'state'
-        if name in ('date', 'state'):
-            return getattr(self.move, name)
-        elif name == 'origin':
-            origin = getattr(self.move, name)
-            if origin:
-                return str(origin)
-            return None
-        else:
-            return getattr(self.move, name).id
+        field = getattr(self.__class__, name)
+        if name.startswith('move_'):
+            name = name[5:]
+        value = getattr(self.move, name)
+        if isinstance(value, ModelSQL):
+            if field._type == 'reference':
+                return str(value)
+            return value.id
+        return value
 
     @classmethod
     def set_move_field(cls, lines, name, value):
-        if name == 'move_state':
-            name = 'state'
+        if name.startswith('move_'):
+            name = name[5:]
         if not value:
             return
         Move = Pool().get('account.move')
@@ -1057,8 +1140,8 @@ class Line(ModelSQL, ModelView):
 
     @classmethod
     def search_move_field(cls, name, clause):
-        if name == 'move_state':
-            name = 'state'
+        if name.startswith('move_'):
+            name = name[5:]
         return [('move.' + name,) + tuple(clause[1:])]
 
     def _order_move_field(name):
@@ -1082,6 +1165,16 @@ class Line(ModelSQL, ModelView):
     order_origin = _order_move_field('origin')
     order_move_state = _order_move_field('state')
 
+    def get_rec_name(self, name):
+        if self.debit > self.credit:
+            return self.account.rec_name
+        else:
+            return '(%s)' % self.account.rec_name
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        return [('account.rec_name',) + tuple(clause[1:])]
+
     @classmethod
     def query_get(cls, table):
         '''
@@ -1188,6 +1281,9 @@ class Line(ModelSQL, ModelView):
         if not self.account.active:
             self.raise_user_error('move_inactive_account', (
                     self.account.rec_name,))
+        if bool(self.party) != bool(self.account.party_required):
+            error = 'party_set' if self.party else 'party_required'
+            self.raise_user_error(error, self.rec_name)
 
     @classmethod
     def check_journal_period_modify(cls, period, journal):
@@ -1213,44 +1309,61 @@ class Line(ModelSQL, ModelView):
                         }])
 
     @classmethod
-    def check_modify(cls, lines):
+    def check_modify(cls, lines, modified_fields=None):
         '''
         Check if the lines can be modified
         '''
+        if (modified_fields is not None
+                and modified_fields <= cls._check_modify_exclude):
+            return
         journal_period_done = []
         for line in lines:
             if line.move.state == 'posted':
                 cls.raise_user_error('modify_posted_move', (
                         line.move.rec_name,))
-            if line.reconciliation:
-                cls.raise_user_error('modify_reconciled', (
-                        line.rec_name,))
             journal_period = (line.journal.id, line.period.id)
             if journal_period not in journal_period_done:
                 cls.check_journal_period_modify(line.period,
                         line.journal)
                 journal_period_done.append(journal_period)
 
+    @classmethod
+    def check_reconciliation(cls, lines, modified_fields=None):
+        if (modified_fields is not None
+                and not modified_fields & cls._reconciliation_modify_disallow):
+            return
+        for line in lines:
+            if line.reconciliation:
+                cls.raise_user_error('modify_reconciled', line.rec_name)
+
     @classmethod
     def delete(cls, lines):
         Move = Pool().get('account.move')
         cls.check_modify(lines)
+        cls.check_reconciliation(lines)
         moves = [x.move for x in lines]
         super(Line, cls).delete(lines)
         Move.validate_move(moves)
 
     @classmethod
-    def write(cls, lines, vals):
+    def write(cls, *args):
         Move = Pool().get('account.move')
 
-        if any(k not in cls._check_modify_exclude for k in vals):
-            cls.check_modify(lines)
-        moves = [x.move for x in lines]
-        super(Line, cls).write(lines, vals)
+        actions = iter(args)
+        args = []
+        moves = []
+        all_lines = []
+        for lines, values in zip(actions, actions):
+            cls.check_modify(lines, set(values.keys()))
+            cls.check_reconciliation(lines, set(values.keys()))
+            moves.extend((x.move for x in lines))
+            all_lines.extend(lines)
+            args.extend((lines, values))
+
+        super(Line, cls).write(*args)
 
         Transaction().timestamp = {}
-
-        Move.validate_move(list(set(l.move for l in lines) | set(moves)))
+        Move.validate_move(list(set(l.move for l in all_lines) | set(moves)))
 
     @classmethod
     def create(cls, vlist):
@@ -1348,7 +1461,8 @@ class Line(ModelSQL, ModelView):
         return result
 
     @classmethod
-    def reconcile(cls, lines, journal=None, date=None, account=None):
+    def reconcile(cls, lines, journal=None, date=None, account=None,
+            description=None):
         pool = Pool()
         Move = pool.get('account.move')
         Reconciliation = pool.get('account.move.reconciliation')
@@ -1361,30 +1475,45 @@ class Line(ModelSQL, ModelView):
                         error_args=(line.move.number, line.id,))
 
         lines = lines[:]
+        reconcile_account = None
+        reconcile_party = None
+        amount = Decimal('0.0')
+        for line in lines:
+            amount += line.debit - line.credit
+            if not reconcile_account:
+                reconcile_account = line.account
+            if not reconcile_party:
+                reconcile_party = line.party
+        amount = reconcile_account.currency.round(amount)
+        if not account and journal:
+            if amount >= 0:
+                account = journal.debit_account
+            else:
+                account = journal.credit_account
         if journal and account:
             if not date:
                 date = Date.today()
-            reconcile_account = None
-            amount = Decimal('0.0')
-            for line in lines:
-                amount += line.debit - line.credit
-                if not reconcile_account:
-                    reconcile_account = line.account
-            amount = reconcile_account.currency.round(amount)
             period_id = Period.find(reconcile_account.company.id, date=date)
             move, = Move.create([{
                         'journal': journal.id,
                         'period': period_id,
                         'date': date,
+                        'description': description,
                         'lines': [
                             ('create', [{
                                         'account': reconcile_account.id,
+                                        'party': (reconcile_party.id
+                                            if reconcile_party else None),
                                         'debit': (amount < Decimal('0.0')
                                             and - amount or Decimal('0.0')),
                                         'credit': (amount > Decimal('0.0')
                                             and amount or Decimal('0.0')),
                                         }, {
                                         'account': account.id,
+                                        'party': (reconcile_party.id
+                                            if (account.party_required and
+                                                reconcile_party)
+                                            else None),
                                         'debit': (amount > Decimal('0.0')
                                             and amount or Decimal('0.0')),
                                         'credit': (amount < Decimal('0.0')
@@ -1530,13 +1659,15 @@ class OpenAccount(Wizard):
 class ReconcileLinesWriteOff(ModelView):
     'Reconcile Lines Write-Off'
     __name__ = 'account.move.reconcile_lines.writeoff'
-    journal = fields.Many2One('account.journal', 'Journal', required=True)
+    journal = fields.Many2One('account.journal', 'Journal', required=True,
+        domain=[
+            ('type', '=', 'write-off'),
+            ])
     date = fields.Date('Date', required=True)
-    account = fields.Many2One('account.account', 'Account', required=True,
-            domain=[('kind', '!=', 'view')])
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
         readonly=True, depends=['currency_digits'])
     currency_digits = fields.Integer('Currency Digits', readonly=True)
+    description = fields.Char('Description')
 
     @staticmethod
     def default_date():
@@ -1587,17 +1718,12 @@ class ReconcileLines(Wizard):
 
         journal = getattr(self.writeoff, 'journal', None)
         date = getattr(self.writeoff, 'date', None)
-        account = getattr(self.writeoff, 'account', None)
+        description = getattr(self.writeoff, 'description', None)
         Line.reconcile(Line.browse(Transaction().context['active_ids']),
-            journal, date, account)
+            journal=journal, date=date, description=description)
         return 'end'
 
 
-class UnreconcileLinesStart(ModelView):
-    'Unreconcile Lines'
-    __name__ = 'account.move.unreconcile_lines.start'
-
-
 class UnreconcileLines(Wizard):
     'Unreconcile Lines'
     __name__ = 'account.move.unreconcile_lines'
@@ -1616,37 +1742,219 @@ class UnreconcileLines(Wizard):
         return 'end'
 
 
-class OpenReconcileLinesStart(ModelView):
-    'Open Reconcile Lines'
-    __name__ = 'account.move.open_reconcile_lines.start'
-    account = fields.Many2One('account.account', 'Account', required=True,
-            domain=[('kind', '!=', 'view'), ('reconcile', '=', True)])
-
-
-class OpenReconcileLines(Wizard):
-    'Open Reconcile Lines'
-    __name__ = 'account.move.open_reconcile_lines'
-    start = StateView('account.move.open_reconcile_lines.start',
-        'account.open_reconcile_lines_start_view_form', [
+class Reconcile(Wizard):
+    'Reconcile'
+    __name__ = 'account.reconcile'
+    start_state = 'next_'
+    next_ = StateTransition()
+    show = StateView('account.reconcile.show',
+        'account.reconcile_show_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Open', 'open_', 'tryton-ok', default=True),
+            Button('Skip', 'next_', 'tryton-go-next'),
+            Button('Reconcile', 'reconcile', 'tryton-ok', default=True),
             ])
-    open_ = StateAction('account.act_move_line_form')
+    reconcile = StateTransition()
 
-    def do_open_(self, action):
-        encoder = PYSONEncoder()
-        action['pyson_domain'] = encoder.encode([
-            ('account', '=', self.start.account.id),
+    def get_accounts(self):
+        'Return a list of account id to reconcile'
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        line = Line.__table__()
+        Account = pool.get('account.account')
+        account = Account.__table__()
+        cursor = Transaction().cursor
+
+        balance = line.debit - line.credit
+        cursor.execute(*line.join(account,
+                condition=line.account == account.id).select(
+                account.id,
+                where=(line.reconciliation == Null) & account.reconcile,
+                group_by=account.id,
+                having=(
+                    Sum(Case((balance > 0, 1), else_=0)) > 0)
+                & (Sum(Case((balance < 0, 1), else_=0)) > 0)
+                ))
+        return [a for a, in cursor.fetchall()]
+
+    def get_parties(self, account):
+        'Return a list party to reconcile for the account'
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        line = Line.__table__()
+        cursor = Transaction().cursor
+
+        balance = line.debit - line.credit
+        cursor.execute(*line.select(line.party,
+                where=(line.reconciliation == Null)
+                & (line.account == account.id),
+                group_by=line.party,
+                having=(
+                    Sum(Case((balance > 0, 1), else_=0)) > 0)
+                & (Sum(Case((balance < 0, 1), else_=0)) > 0)
+                ))
+        return [p for p, in cursor.fetchall()]
+
+    def transition_next_(self):
+
+        def next_account():
+            accounts = list(self.show.accounts)
+            if not accounts:
+                return
+            account = accounts.pop()
+            self.show.account = account
+            self.show.parties = self.get_parties(account)
+            self.show.accounts = accounts
+            return account
+
+        def next_party():
+            parties = list(self.show.parties)
+            if not parties:
+                return
+            party = parties.pop()
+            self.show.party = party
+            self.show.parties = parties
+            return party
+
+        if getattr(self.show, 'accounts', None) is None:
+            self.show.accounts = self.get_accounts()
+            if not next_account():
+                return 'end'
+        if getattr(self.show, 'parties', None) is None:
+            self.show.parties = self.get_parties(self.show.account)
+
+        while not next_party():
+            if not next_account():
+                return 'end'
+        return 'show'
+
+    def default_show(self, fields):
+        pool = Pool()
+        Date = pool.get('ir.date')
+
+        defaults = {}
+        defaults['accounts'] = [a.id for a in self.show.accounts]
+        defaults['account'] = self.show.account.id
+        defaults['parties'] = [p.id for p in self.show.parties]
+        defaults['party'] = self.show.party.id if self.show.party else None
+        defaults['currency_digits'] = self.show.account.company.currency.digits
+        defaults['lines'] = self._default_lines()
+        defaults['write_off'] = Decimal(0)
+        defaults['date'] = Date.today()
+        return defaults
+
+    def _all_lines(self):
+        'Return all lines to reconcile for the current state'
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        return Line.search([
+                ('account', '=', self.show.account.id),
+                ('party', '=',
+                    self.show.party.id if self.show.party else None),
+                ('reconciliation', '=', None),
+                ])
+
+    def _default_lines(self):
+        'Return the larger list of lines which can be reconciled'
+        currency = self.show.account.company.currency
+        chunk = config.getint('account', 'reconciliation_chunk', 10)
+        # Combination is exponential so it must be limited to small number
+        default = []
+        for lines in grouped_slice(self._all_lines(), chunk):
+            lines = list(lines)
+            best = None
+            for n in xrange(len(lines), 1, -1):
+                for comb_lines in combinations(lines, n):
+                    amount = sum((l.debit - l.credit) for l in comb_lines)
+                    if currency.is_zero(amount):
+                        best = [l.id for l in comb_lines]
+                        break
+                if best:
+                    break
+            if best:
+                default.extend(best)
+        return default
+
+    def transition_reconcile(self):
+        pool = Pool()
+        Line = pool.get('account.move.line')
+
+        if self.show.lines:
+            Line.reconcile(self.show.lines,
+                journal=self.show.journal,
+                date=self.show.date,
+                description=self.show.description)
+        return 'next_'
+
+
+class ReconcileShow(ModelView):
+    'Reconcile'
+    __name__ = 'account.reconcile.show'
+    accounts = fields.One2Many('account.account', None, 'Account',
+        readonly=True)
+    account = fields.Many2One('account.account', 'Account', readonly=True)
+    parties = fields.One2Many('party.party', None, 'Parties', readonly=True)
+    party = fields.Many2One('party.party', 'Party', readonly=True)
+    lines = fields.Many2Many('account.move.line', None, None, 'Lines',
+        domain=[
+            ('account', '=', Eval('account')),
+            ('party', '=', Eval('party')),
             ('reconciliation', '=', None),
+            ],
+        depends=['account', 'party'])
+
+    _write_off_states = {
+        'required': Bool(Eval('write_off', 0)),
+        'invisible': ~Eval('write_off', 0),
+        }
+    _write_off_depends = ['write_off']
+
+    write_off = fields.Function(fields.Numeric('Write-Off',
+            digits=(16, Eval('currency_digits', 2)),
+            states=_write_off_states,
+            depends=_write_off_depends + ['currency_digits']),
+        'on_change_with_write_off')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
+    journal = fields.Many2One('account.journal', 'Journal',
+        states=_write_off_states, depends=_write_off_depends,
+        domain=[
+            ('type', '=', 'write-off'),
             ])
-        if self.start.account.kind == 'receivable':
-            order = [('credit', 'DESC'), ('id', 'DESC')]
-        elif self.start.account.kind == 'payable':
-            order = [('debit', 'DESC'), ('id', 'DESC')]
-        else:
-            order = None
-        action['pyson_order'] = encoder.encode(order)
-        return action, {}
+    date = fields.Date('Date',
+        states=_write_off_states, depends=_write_off_depends)
+    description = fields.Char('Description',
+        states=_write_off_states, depends=_write_off_depends)
+
+    @fields.depends('lines')
+    def on_change_with_write_off(self, name=None):
+        return sum((l.debit - l.credit) for l in self.lines)
+
+    @fields.depends('account')
+    def on_change_with_currency_digits(self, name=None):
+        return self.account.company.currency.digits
+
+
+class CancelMoves(Wizard):
+    'Cancel Moves'
+    __name__ = 'account.move.cancel'
+    start_state = 'cancel'
+    cancel = StateTransition()
+
+    def transition_cancel(self):
+        pool = Pool()
+        Move = pool.get('account.move')
+        Line = pool.get('account.move.line')
+
+        moves = Move.browse(Transaction().context['active_ids'])
+        for move in moves:
+            cancel_move = move.cancel()
+            to_reconcile = defaultdict(list)
+            for line in move.lines + cancel_move.lines:
+                if line.account.reconcile:
+                    to_reconcile[line.account].append(line)
+            for lines in to_reconcile.itervalues():
+                Line.reconcile(lines)
+        return 'end'
 
 
 class FiscalYearLine(ModelSQL):
